##
##  etcd.spec -- OpenPKG RPM Package Specification
##  Copyright (c) 2000-2024 OpenPKG Project <http://openpkg.org/>
##
##  Permission to use, copy, modify, and distribute this software for
##  any purpose with or without fee is hereby granted, provided that
##  the above copyright notice and this permission notice appear in all
##  copies.
##
##  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
##  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
##  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
##  IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS AND THEIR
##  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
##  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
##  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
##  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
##  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
##  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
##  OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
##  SUCH DAMAGE.
##

#   package version
%define       V_base 3.5.15
%define       V_snap 20240720

#   package information
Name:         etcd
Summary:      High-Availability Distributed Key/Value Store
URL:          http://etcd.io/
Vendor:       Cloud Native Computing Foundation (CNCF)
Packager:     OpenPKG Project
Distribution: OpenPKG Community
Class:        EVAL
Group:        Networking
License:      Apache
Version:      %{V_base}.%{V_snap}
Release:      20240720

#   list of sources
Source0:      http://download.openpkg.org/components/versioned/etcd/etcd-%{V_snap}.tar.xz
Source1:      etcd.conf
Source2:      rc.etcd

#   build information
BuildPreReq:  OpenPKG, openpkg >= 20160101, go
PreReq:       OpenPKG, openpkg >= 20160101

%description
    EtcD is a highly-available key/value store for shared configuration
    and service discovery of a cluster, based on the RAFT consensus
    protocol to manage a highly-available replicated log. EtcD is
    usually used as the cluster manager for distributed applications.

%track
    prog etcd:base = {
        version   = %{V_base}
        url       = https://github.com/etcd-io/etcd/releases
        regex     = tree/v(\d+\.\d+\.\d+)
    }
    prog etcd:snap = {
        version   = %{V_snap}
        url       = https://github.com/etcd-io/etcd/releases
        regex     = v(__VER__)\.tar\.gz
    }

%prep
    %setup -q -n etcd

%build
    #   build program
    export GOPATH=`pwd`
    cd src/github.com/etcd-io/etcd
    go build -v -o bin/etcd .
    cd etcdctl
    go build -v -o ../bin/etcdctl main.go

%install
    #   create directory hierarchy
    %{l_shtool} mkdir -f -p -m 755 \
        $RPM_BUILD_ROOT%{l_prefix}/sbin \
        $RPM_BUILD_ROOT%{l_prefix}/etc/rc.d \
        $RPM_BUILD_ROOT%{l_prefix}/etc/etcd \
        $RPM_BUILD_ROOT%{l_prefix}/var/etcd/log \
        $RPM_BUILD_ROOT%{l_prefix}/var/etcd/run \
        $RPM_BUILD_ROOT%{l_prefix}/var/etcd/data \
        $RPM_BUILD_ROOT%{l_prefix}/var/etcd/wal

    #   install program
    %{l_shtool} install -c -s -m 755 \
        src/github.com/etcd-io/etcd/bin/etcd \
        src/github.com/etcd-io/etcd/bin/etcdctl \
        $RPM_BUILD_ROOT%{l_prefix}/sbin/

    #   install default configutation
    %{l_shtool} install -c -m 644 %{l_value -s -a} \
        %{SOURCE etcd.conf} $RPM_BUILD_ROOT%{l_prefix}/etc/etcd/

    #   install run-command script
    %{l_shtool} install -c -m 755 %{l_value -s -a} \
        %{SOURCE rc.etcd} $RPM_BUILD_ROOT%{l_prefix}/etc/rc.d/

    #   determine installation files
    %{l_rpmtool} files -v -ofiles -r$RPM_BUILD_ROOT \
        %{l_files_std} \
        '%config %{l_prefix}/etc/etcd/*' \
        '%attr(-,%{l_rusr},%{l_rgrp}) %{l_prefix}/var/etcd' \
        '%attr(-,%{l_rusr},%{l_rgrp}) %{l_prefix}/var/etcd/*'

%files -f files

%clean

%post
    if [ $1 -eq 1 ]; then
        #   display information about next steps
        ( echo "You are advised to use authentication for EtcD. For this,"
          echo "after starting EtcD with..."
          echo "    \$ $RPM_INSTALL_PREFIX/bin/openpkg rc etcd start"
          echo "...create the special user \"root\" and enable authentication with:"
          echo "    \$ $RPM_INSTALL_PREFIX/sbin/etcdctl user add root:<password>"
          echo "    \$ $RPM_INSTALL_PREFIX/sbin/etcdctl user grant-role root root"
          echo "    \$ $RPM_INSTALL_PREFIX/sbin/etcdctl auth enable"
        ) | %{l_rpmtool} msg -b -t notice
    elif [ $1 -eq 2 ]; then
        #   after upgrade, restart service
        eval `%{l_rc} etcd status 2>/dev/null`
        [ ".$etcd_active" = .yes ] && %{l_rc} etcd restart
    fi
    exit 0

%preun
    if [ $1 -eq 0 ]; then
        #   before erase, stop service and remove log files
        %{l_rc} etcd stop 2>/dev/null
        rm -rf $RPM_INSTALL_PREFIX/var/etcd/log/*  >/dev/null 2>&1 || true
        rm -rf $RPM_INSTALL_PREFIX/var/etcd/run/*  >/dev/null 2>&1 || true
        rm -rf $RPM_INSTALL_PREFIX/var/etcd/data/* >/dev/null 2>&1 || true
        rm -rf $RPM_INSTALL_PREFIX/var/etcd/wal/*  >/dev/null 2>&1 || true
    fi
    exit 0

